function [w, T, bias] = SCut(train_data, options, fbr, train_final)
    % Apply Algorithm 1 in the paper to every label with F-measure.
    % FBR heuristic can also be used.
    % By default fbr=-1, which means fbr heuristic is not used

    % check input
    if (nargin ~= 4)
        if (nargin < 4)
            train_final = true;
        end
        if (nargin < 3)
            fbr = -1;
        end
        if (nargin < 2)
            options = '-q -B 1';
        end
        if (nargin < 1 || nargin > 4)
            fprintf('Incorrect number of input parameters\n');
            return;
        end
    end
    
    x = train_data.x;
    y = train_data.y;
    map = train_data.map;
        
    % bias option
    bias = get_bias(options);
    if bias > 0
        w_size = size(x, 1) + 1;
    else
        w_size = size(x, 1);
    end

    t0all = clock;
    
    % number of label = number of model to be trained
    L = size(y, 2); 
    
    w = zeros(w_size, L);
    T = zeros(L, 1);

    % obtain (w,T) for each label
    parfor i = 1:L
        fprintf(1, 'INFO: training %dth label (%d)...\n', i, map(i));
        t0 = clock;
        % do not calculate y*2 - 1 at once
        [w(:, i), T(i)] = scutfbr(y(:, i)*2 - 1, x, fbr, options, train_final);
        % fprintf(1, 'INFO: b = %g\n', b(i));
        fprintf(1, '\tINFO: Done in %.6f seconds\n', etime(clock, t0));
    end

    fprintf(1, 'INFO: Total Time: %.6f seconds\n', etime(clock, t0all));
end