
function [micro, macro] = f1_score(ty, pre_all, tmap, map, verbose)

    % check input
    if (nargin ~= 5)
        if (nargin == 4)
            verbose = false;
        else
            fprintf('Incorrect number of input parameters\n');
            return;
        end
    end

    assert(size(map,1) == size(pre_all,2), 'Number of labels in map mismatched with prediction.')
    d = size(map,1); % number of labels appeared in training set
    pre_all = pre_all > 0;
    % allow y to be +1,-1 or 1,0
    ty = ty > 0;

    F = 0;
    tp_sum = 0;
    fp_sum = 0;
    fn_sum = 0;
    tn_sum = 0;
    
    for i = 1:d
        % deal with this label
        label = map(i);
        pre = pre_all(:,i);
        test_idx = find(tmap == label);

        if isempty(test_idx)
            orig = zeros(size(ty, 1), 1);
        else
            orig = full(ty(:, test_idx));
        end


        tp = sum(orig == +1 & pre == +1);
        fn = sum(orig == +1 & pre == 0);

        tn = sum(orig == 0 & pre == 0);
        fp = sum(orig == 0 & pre == +1);

        this_F = 0;
        if tp ~= 0 || fp ~= 0 || fn ~= 0
            % calculate F_1 score
            this_F = (2*tp) / (2*tp + fp + fn);
        end
        F = F + this_F;

        tp_sum = tp_sum + tp;
        fp_sum = fp_sum + fp;
        fn_sum = fn_sum + fn;
        tn_sum = tn_sum + tn;

        if verbose
            fprintf(1, 'INFO: %dth label %d:\tF %.6f\ttp %d\tfp %d\ttn %d\tfn %d\n', i, label, this_F, tp, fp, tn, fn);
        end
    end

    micro = (2*tp_sum) / (2*tp_sum + fp_sum + fn_sum);
    macro = F / d;

    if verbose
        fprintf(1, 'INFO: tp_sum %d fp_sum %d fn_sum %d tn_sum %d\n', tp_sum, fp_sum, fn_sum, tn_sum);
        fprintf(1, 'INFO: microaverage: %.6f\n', micro);
        fprintf(1, 'INFO: macroaverage: %.6f\n', macro);
    end
end